! *************************************************************************************************
! sue_lib.f90
! library constants and routines
! *************************************************************************************************



MODULE sue_lib
  

  USE sue_config
  IMPLICIT NONE
  SAVE

 
! *************************************************************************************************
! *** PARAMETERS **********************************************************************************
! *************************************************************************************************


! set the maximum real number precision to be used
  INTEGER,PARAMETER::fp_1 = SELECTED_REAL_KIND(P=14)


! *** model configuration constants ***
! array dimensions 
  INTEGER,PARAMETER::n_tracer           = 13      ! number of ocean tracer physical descriptors
  INTEGER,PARAMETER::n_atm              = 04      ! number of atmospheric tracers
  INTEGER,PARAMETER::n_phys             = 29      ! number of ocean box physical descriptors
  INTEGER,PARAMETER::n_phys_kyr         = 10      ! number of ocean box physical descriptors (saved kyr data)
  INTEGER,PARAMETER::n_phys_spin        = 10      ! number of ocean box physical descriptors (saved spin data)
  INTEGER,PARAMETER::n_dis              = 10      ! 
  INTEGER,PARAMETER::n_bionp            = 24      ! number of biological new production descriptors
  INTEGER,PARAMETER::n_sed_fnp          = 11      ! flux of new production to sediments
  INTEGER,PARAMETER::n_bath_depth       = 6000    ! ocean depth array (NOT 10000 any longer)
  INTEGER,PARAMETER::n_bath_par         = 02      !
  INTEGER,PARAMETER::n_biok             = 12      ! number of new production kinetic scalars
  INTEGER,PARAMETER::n_sed_par          = 32      ! number of sediment layer descriptors 
  INTEGER,PARAMETER::n_sed_tot          = 250     ! number of sedimentary stack sub-layers
  INTEGER,PARAMETER::n_sed_tot_init     = 050     ! initial number of sedimentary stack sub-layers filled
  INTEGER,PARAMETER::n_sed_com_par      = 16      ! number of general sediment composition descriptors
  INTEGER,PARAMETER::n_sedcor_max       = 999     ! maximum allowable number of user-defined sed cores
  INTEGER,PARAMETER::n_sedcor_tot_init  = 050     ! initial number of sediment core stack sub-layers filled
  INTEGER,PARAMETER::n_wflux_max        = 9999    ! maximum alowed number of water mass flux vectors
  INTEGER,PARAMETER::n_sscc             = 04      ! number of sea surface carbonate chemistry descriptors
  INTEGER,PARAMETER::n_col_max          = 99      ! maximum number of allowable ocean columns
  INTEGER,PARAMETER::n_t_np_max         = 360     ! maximum number of np time steps
  INTEGER,PARAMETER::n_t_np_tot         = 720     ! total size of np time step array
  INTEGER,PARAMETER::n_t_int_max        = 24      ! maximum number of int time steps

! *** model option arrays ***
! program run options matrix
  CHARACTER(LEN=1),DIMENSION(19)::runopt
! forcing options matrix
  CHARACTER(LEN=1),DIMENSION(16)::forcingopt
! biological new production and remineralization options
  CHARACTER(LEN=1),DIMENSION(16)::bioopt
! sediment sub-system options matrix
  CHARACTER(LEN=1),DIMENSION(20)::sedopt
! gaian meta-system options matrix
  CHARACTER(LEN=1),DIMENSION(5)::gaiaopt
! misc options matrix
  CHARACTER(LEN=1),DIMENSION(7)::miscopt
  
! *** program option matrix indices ***
! program run options indices
  INTEGER,PARAMETER::ropt_main                    = 1     ! 
  INTEGER,PARAMETER::ropt_bio                     = 2     ! 
  INTEGER,PARAMETER::ropt_sed                     = 3     ! 
  INTEGER,PARAMETER::ropt_gaia                    = 4     ! 
  INTEGER,PARAMETER::ropt_specmap                 = 5     ! 
  INTEGER,PARAMETER::ropt_future                  = 6     ! 
  INTEGER,PARAMETER::ropt_sedres                  = 7     ! 
  INTEGER,PARAMETER::ropt_sedcor                  = 8     ! 
  INTEGER,PARAMETER::ropt_sedcor_sedrate          = 9     ! 
  INTEGER,PARAMETER::ropt_sed_fraccal             = 10    ! 
  INTEGER,PARAMETER::ropt_debug_main              = 11    ! 
  INTEGER,PARAMETER::ropt_debug_box               = 12    ! 
  INTEGER,PARAMETER::ropt_debug_sed_lib           = 13    ! 
  INTEGER,PARAMETER::ropt_debug_sed               = 14    ! 
  INTEGER,PARAMETER::ropt_warn                    = 15    ! 
  INTEGER,PARAMETER::ropt_save_box_data_ind       = 16    !
  INTEGER,PARAMETER::ropt_load_modelstate         = 17    ! load model state?
  INTEGER,PARAMETER::ropt_save_modelstate         = 18    ! save model state?
  INTEGER,PARAMETER::ropt_chain_modelstate        = 19    ! automatic save/load (chain) model state?
! forcing options indices
  INTEGER,PARAMETER::fopt_V                       = 01    ! 
  INTEGER,PARAMETER::fopt_A                       = 02    ! 
  INTEGER,PARAMETER::fopt_SST                     = 03    ! 
  INTEGER,PARAMETER::fopt_sig_swind               = 04    ! 
  INTEGER,PARAMETER::fopt_sig_sealevel            = 05    ! 
  INTEGER,PARAMETER::fopt_sig_SST                 = 06    !  
  INTEGER,PARAMETER::fopt_sig_SSS                 = 07    !  
  INTEGER,PARAMETER::fopt_sig_I                   = 08    ! 
  INTEGER,PARAMETER::fopt_sig_seaice              = 09    !
  INTEGER,PARAMETER::fopt_sig_terr_C              = 10    ! 
  INTEGER,PARAMETER::fopt_sig_neri_CaCO3          = 11    ! use neritic sediment CaCO3 forcing signal
  INTEGER,PARAMETER::fopt_sig_neri_opal           = 12    ! use neritic sediment opal forcing signal
  INTEGER,PARAMETER::fopt_sig_fdust               = 13    ! terregeneous dust deposition signal 
  INTEGER,PARAMETER::fopt_sig_SSPO4               = 14    !  
  INTEGER,PARAMETER::fopt_sig_ocir                = 15    !  
  INTEGER,PARAMETER::fopt_sig_fSi                 = 16    !  
! biological new production and remineralization options
  INTEGER,PARAMETER::bopt_remin_Corg_alt          = 01    ! Corg remineralization scheme
  INTEGER,PARAMETER::bopt_remin_cal_alt           = 16    ! arg remineralization scheme
  INTEGER,PARAMETER::bopt_remin_arg_alt           = 02    ! arg remineralization scheme
  INTEGER,PARAMETER::bopt_remin_opal_alt          = 03    ! opal remimineralization scheme
  INTEGER,PARAMETER::bopt_d13Corg_alt             = 04    ! DIC:Corg 13C fractionation scheme (1-2)
  INTEGER,PARAMETER::bopt_d13CaCO3_fp             = 05    ! planktonic foraminiferal 13C DIC->CaCO3 scheme
  INTEGER,PARAMETER::bopt_d13CaCO3_fb             = 06    ! benthic foraminiferal 13C DIC->CaCO3 scheme
  INTEGER,PARAMETER::bopt_arg_remin               = 07    ! remineralize aragonite in water column?
  INTEGER,PARAMETER::bopt_scav_Fe_alt             = 08    ! Fe scaveging scheme (1-3)
  INTEGER,PARAMETER::bopt_np_alt                  = 09    ! new production scheme (1-4)
  INTEGER,PARAMETER::bopt_np_Tlaw                 = 10    ! temperature-dependent new production kinetics?
  INTEGER,PARAMETER::bopt_np_fixed_FetoC_dia      = 11    ! fixed Fe:C ratio for diatoms?
  INTEGER,PARAMETER::bopt_np_fixed_FetoC_coc      = 12    ! fixed Fe:C ratio forcoccoliths?
  INTEGER,PARAMETER::bopt_np_fixed_Corgtoopal_dia = 13    ! fixed Norg:opal ratio for diatoms?
  INTEGER,PARAMETER::bopt_np_latinsol             = 14    ! latitude-dependent insolation function?
  INTEGER,PARAMETER::bopt_uniform_frac_Fe_dis     = 15    ! use spatially uniform Fe solubility fraction?
! sediment sub-system options
  INTEGER,PARAMETER::sopt_sed_mix                 = 1   ! mix the sediment stack sub-layers?
  INTEGER,PARAMETER::sopt_sedcor_mix              = 2   ! 
  INTEGER,PARAMETER::sopt_disfraccal              = 3   ! 
  INTEGER,PARAMETER::sopt_mix_homog               = 4   ! 
  INTEGER,PARAMETER::sopt_sed_save_wtper          = 6   ! save sediment data as wt% (otherwise vol%)
  INTEGER,PARAMETER::sopt_sedcor_fixedacc         = 7   ! fixed sedimentation rate (& no diagenesis)?
  INTEGER,PARAMETER::sopt_save_sedsysdata         = 8   ! save sediment sub-system core data?
  INTEGER,PARAMETER::sopt_sed_rdis_fixed          = 9   ! fixed inter./homog. sediment dissolution ratio 
  INTEGER,PARAMETER::sopt_sed_14C                 = 10  ! decay 14C in sediments?
  INTEGER,PARAMETER::sopt_sedcor_14C              = 11  ! decay 14C in sediments?
  INTEGER,PARAMETER::sopt_sed_diagen_respir       = 12  ! allow respiration-driven CaCO3 dissolution?
  INTEGER,PARAMETER::sopt_save_sedfnpdata         = 13  ! save sediment sub-system water->sediment flux data?
  INTEGER,PARAMETER::sopt_save_sedcorfnpdata      = 14  ! save user-definable core water->sediment flux data?
  INTEGER,PARAMETER::sopt_save_seddisdata         = 16  ! save sediment sub-system sediment->water flux data?
  INTEGER,PARAMETER::sopt_save_sedcordisdata      = 17  ! save user-definable core sediment->water flux data?
  INTEGER,PARAMETER::sopt_sed_diagen_lookup_CaCO3 = 5   ! use lookup table CaCO3 sediment diagenesis scheme?
  INTEGER,PARAMETER::sopt_sed_diagen_lookup_opal  = 15  ! use lookup table opal sediment diagenesis scheme?
  INTEGER,PARAMETER::sopt_sed_diagen_AltoasymSi   = 18  ! asymptotic [Si] dependence on %refrac/%opal?
  INTEGER,PARAMETER::sopt_sed_diagen_AltoKSi      = 19  ! KSi dependence on %refrac/%opal?
  INTEGER,PARAMETER::sopt_save_sed_data_ind       = 20  ! 
! gaian meta-system options
  INTEGER,PARAMETER::gopt_ext_sig_Cterr           = 01  ! use external terrestrial carbon storage signal?
  INTEGER,PARAMETER::gopt_ext_sig_CaCO3           = 02  ! use external neritic CaCO3 storage signals?
  INTEGER,PARAMETER::gopt_ext_sig_opal            = 03  ! use external neritic opal storage signal?
  INTEGER,PARAMETER::gopt_neri_CaCO3_reefal_dsealevel = 04  !
  INTEGER,PARAMETER::gopt_uniform_frac_SiO2_dis   = 05  ! use spatially uniform SiO2 solubility fraction?
! misc options
  INTEGER,PARAMETER::mopt_uniform_kgastrans       = 01  ! use uniform (fixed) air-sea exchange rate?
  INTEGER,PARAMETER::mopt_uniform_vdif            = 02  ! use uniform vertical eddy diffusivity coefficient?
  INTEGER,PARAMETER::mopt_d13atm_alt              = 03  ! alternative sea-air-sea 13C fractionation number
  INTEGER,PARAMETER::mopt_conv_adj                = 04  ! ocean convective adjustment scheme
  INTEGER,PARAMETER::mopt_Alatlon                 = 05  ! calculate ocean surface area from lat-lon data?
  INTEGER,PARAMETER::mopt_co2calc_alt             = 06  ! alternative sea-air-sea 13C fractionation number
  INTEGER,PARAMETER::mopt_conv_dw_maxD            = 07  ! force maximum ocean depth for deepwater penetration?

! *** parameter matrix indices ***
! physics array indices
  INTEGER,PARAMETER::ip_V                 = 01    ! ocean box volume (m3)
  INTEGER,PARAMETER::ip_M                 = 02    ! ocean box water mass (kg)
  INTEGER,PARAMETER::ip_D_top             = 03    ! 
  INTEGER,PARAMETER::ip_D_bot             = 04    ! 
  INTEGER,PARAMETER::ip_A_top             = 05    ! 
  INTEGER,PARAMETER::ip_A_bot             = 06    ! 
  INTEGER,PARAMETER::ip_csA_N             = 07    ! cross-sectional area (northwards buondary)
  INTEGER,PARAMETER::ip_csA_S             = 08    ! cross-sectional area (southwards buondary)
  INTEGER,PARAMETER::ip_D_calsat          = 09    ! 
  INTEGER,PARAMETER::ip_D_argsat          = 10    ! 
  INTEGER,PARAMETER::ip_reminf_Corg       = 11    ! value of remineralization function at lower layer depth
  INTEGER,PARAMETER::ip_reminf_opal       = 12    ! value of remineralization function at lower layer depth
  INTEGER,PARAMETER::ip_reminf_cal        = 13    ! value of remineralization function at lower layer depth
  INTEGER,PARAMETER::ip_reminf_arg        = 14    ! value of remineralization function at lower layer depth
  INTEGER,PARAMETER::ip_T                 = 15    ! ocean box temperature (K)
  INTEGER,PARAMETER::ip_H                 = 16    ! hydrogen ion concentration
  INTEGER,PARAMETER::ip_vdif              = 17    ! vertical diffusivity
  INTEGER,PARAMETER::ip_conv              = 18    ! convective adjustment
  INTEGER,PARAMETER::ip_swind             = 19    ! mean surface wind speed
  INTEGER,PARAMETER::ip_I                 = 20    ! surface solar insolation factor
  INTEGER,PARAMETER::ip_lat               = 21    ! latitude (mid-point)
  INTEGER,PARAMETER::ip_lon               = 22    ! longitude (mid-point)
  INTEGER,PARAMETER::ip_dlat              = 23    ! latitude (width)
  INTEGER,PARAMETER::ip_dlon              = 24    ! longitude (width)
  INTEGER,PARAMETER::ip_seaice_min        = 25    ! fractional sea ice coverage (summer minimum)
  INTEGER,PARAMETER::ip_seaice_max        = 26    ! fractional sea ice coverage (winter maximum)
  INTEGER,PARAMETER::ip_conv_dw           = 27    ! deep-water formation factor
  INTEGER,PARAMETER::ip_conv_mixD         = 28    ! maximum allowed mixed layer depth (m)
  INTEGER,PARAMETER::ip_remin_topD        = 29    ! top remineralization depth
! dissociation constant array indices
  INTEGER,PARAMETER::idis_k               = 01    ! 
  INTEGER,PARAMETER::idis_k1              = 02    ! 
  INTEGER,PARAMETER::idis_k2              = 03    ! 
  INTEGER,PARAMETER::idis_kB              = 04    ! 
  INTEGER,PARAMETER::idis_kW              = 05    ! 
  INTEGER,PARAMETER::idis_kSi             = 06    ! 
  INTEGER,PARAMETER::idis_kcal            = 07    ! 
  INTEGER,PARAMETER::idis_karg            = 08    ! 
  INTEGER,PARAMETER::idis_QCO2            = 09    ! 
  INTEGER,PARAMETER::idis_QO2             = 10    ! 
! tracer array indices
  INTEGER,PARAMETER::it_CO2               = 01    ! ocean [CO2]
  INTEGER,PARAMETER::it_13C_CO2           = 02    ! ocean [13C(CO2)]
  INTEGER,PARAMETER::it_O2                = 03    ! ocean [O2]
  INTEGER,PARAMETER::it_18O_O2            = 04    ! ocean [18O(O2)]
  INTEGER,PARAMETER::it_PO4               = 05    ! ocean [PO4]
  INTEGER,PARAMETER::it_alk               = 06    ! ocean [alkalinity]
  INTEGER,PARAMETER::it_Si                = 07    ! ocean [Si]
  INTEGER,PARAMETER::it_S                 = 08    ! ocean salinity
  INTEGER,PARAMETER::it_Fe                = 09    ! ocean [Fe]
  INTEGER,PARAMETER::it_T                 = 10    ! ocean temperature (as a density)
  INTEGER,PARAMETER::it_DOC               = 11    ! ocean [DOC]
  INTEGER,PARAMETER::it_13C_DOC           = 12    ! ocean [13C(DOC)]
  INTEGER,PARAMETER::it_DOFe              = 13    ! ocean [DOFe]
! atmospheric tracer indices
  INTEGER,PARAMETER::ia_pCO2              = 01    ! atmosphere pCO2
  INTEGER,PARAMETER::ia_13C_pCO2          = 02    ! atmosphere d13C
  INTEGER,PARAMETER::ia_pO2               = 03    ! atmosphere pO2
  INTEGER,PARAMETER::ia_18O_pO2           = 04    ! atmosphere d18O
! new production flux to sediment indices
  INTEGER,PARAMETER::ised_fnp_Corg        = 01    ! organic carbon flux to sediments 
  INTEGER,PARAMETER::ised_fnp_Porg        = 02    ! PO4 flux to sediments
  INTEGER,PARAMETER::ised_fnp_Norg        = 03    ! NO3 flux to sediments 
  INTEGER,PARAMETER::ised_fnp_Feorg       = 04    ! Fe flux to sediments 
  INTEGER,PARAMETER::ised_fnp_CaCO3       = 05    ! CaCO3 flux to sediments 
  INTEGER,PARAMETER::ised_fnp_cal         = 06    ! calcite flux to sediments  
  INTEGER,PARAMETER::ised_fnp_arg         = 07    ! aragonite flux to sediments  
  INTEGER,PARAMETER::ised_fnp_opal        = 08    ! opal flux to sediments  
  INTEGER,PARAMETER::ised_fnp_13C_Corg    = 09    ! 13C POC flux to sediments  
  INTEGER,PARAMETER::ised_fnp_13C_cal     = 10    ! 13C calcite flux to sediments 
  INTEGER,PARAMETER::ised_fnp_13C_arg     = 11    ! 13C aragonite flux to sediments 
! new production
  INTEGER,PARAMETER::ibionp_Porg_tot      = 01    ! PO4 new production POM - total
  INTEGER,PARAMETER::ibionp_Porg_dia      = 02    ! PO4 new production POM - diatom-based
  INTEGER,PARAMETER::ibionp_Porg_coc      = 03    ! PO4 new production POM - coccolithophore-based
  INTEGER,PARAMETER::ibionp_Corg_dia      = 04    ! C new production POM - diatom-based
  INTEGER,PARAMETER::ibionp_Corg_coc      = 05    ! C new production POM - coccolithophore-based
  INTEGER,PARAMETER::ibionp_Corg_tot      = 06    ! C new production POM - total
  INTEGER,PARAMETER::ibionp_Norg_tot      = 07    ! N new production POM - total
  INTEGER,PARAMETER::ibionp_Feorg_tot     = 08    ! Fe new production POM - total
  INTEGER,PARAMETER::ibionp_CaCO3_coc     = 09    ! carbonate new production - coccolithophore-based
  INTEGER,PARAMETER::ibionp_cal_coc       = 10    ! calcite new production - coccolithophore-based
  INTEGER,PARAMETER::ibionp_arg_coc       = 11    ! aragonite new production - coccolithophore-based
  INTEGER,PARAMETER::ibionp_opal_dia      = 12    ! opal new production - diatom-based
  INTEGER,PARAMETER::ibionp_13C_Corg_dia  = 13    !
  INTEGER,PARAMETER::ibionp_13C_Corg_coc  = 14    !
  INTEGER,PARAMETER::ibionp_13C_Corg_tot  = 15    !
  INTEGER,PARAMETER::ibionp_13C_cal_coc   = 16    !
  INTEGER,PARAMETER::ibionp_13C_arg_coc   = 17    !
  INTEGER,PARAMETER::ibionp_DOC_dia       = 18    !
  INTEGER,PARAMETER::ibionp_DOC_coc       = 19    !
  INTEGER,PARAMETER::ibionp_DOC_tot       = 20    !
  INTEGER,PARAMETER::ibionp_13C_DOC_dia   = 21    !
  INTEGER,PARAMETER::ibionp_13C_DOC_coc   = 22    !
  INTEGER,PARAMETER::ibionp_13C_DOC_tot   = 23    !
  INTEGER,PARAMETER::ibionp_DOFe_tot      = 24    !
! new production kinetic scalars
  INTEGER,PARAMETER::ibiok_I_coc          = 01    ! solar insolation (coccolithophores)
  INTEGER,PARAMETER::ibiok_I_dia          = 02    ! solar insolation (diatoms)
  INTEGER,PARAMETER::ibiok_T_coc          = 03    ! temperature (coccolithophores)
  INTEGER,PARAMETER::ibiok_T_dia          = 04    ! temperature (diatoms)
  INTEGER,PARAMETER::ibiok_PO4_dia        = 05    ! [PO4] (diatoms)
  INTEGER,PARAMETER::ibiok_PO4_coc        = 06    ! [PO4] (coccolithophores)
  INTEGER,PARAMETER::ibiok_Si_dia         = 07    ! [H4SiO4] (diatoms)
  INTEGER,PARAMETER::ibiok_Si_coc         = 08    ! [H4SiO4] (coccolithophores)
  INTEGER,PARAMETER::ibiok_Fe_dia         = 09    ! [Fe] (diatoms)
  INTEGER,PARAMETER::ibiok_Fe_coc         = 10    ! [Fe] (coccolithophores)
  INTEGER,PARAMETER::ibiok_tot_dia        = 11    ! total (diatoms)
  INTEGER,PARAMETER::ibiok_tot_coc        = 12    ! total (coccolithophores)
! primary production biomass
  INTEGER,PARAMETER::ibiomass_dia_n       = 01    ! number of diatom cells (l-1)
  INTEGER,PARAMETER::ibiomass_coc_n       = 02    ! number of diatom cells (l-1)
! sediment system - parameters of each sedimentary depth layer
  INTEGER,PARAMETER::ised_par_colnum      = 01    ! 
  INTEGER,PARAMETER::ised_par_D_top       = 02    ! 
  INTEGER,PARAMETER::ised_par_D_bot       = 03    ! 
  INTEGER,PARAMETER::ised_par_D_mid       = 04    ! 
  INTEGER,PARAMETER::ised_par_D_PIG_top   = 05    ! 
  INTEGER,PARAMETER::ised_par_D_PIG_bot   = 06    ! 
  INTEGER,PARAMETER::ised_par_D_PIG_mid   = 07    ! 
  INTEGER,PARAMETER::ised_par_lay_top     = 08    ! top ocean layer adjoining sedimentary layer
  INTEGER,PARAMETER::ised_par_lay_bot     = 09    ! bottom ocean layer adjoining sedimentary layer
  INTEGER,PARAMETER::ised_par_A           = 10    ! area of sedimentary layer (m2)
  INTEGER,PARAMETER::ised_par_Afrac       = 11    ! area fraction of sedimentary layer (of PIG total area)
  INTEGER,PARAMETER::ised_par_Afrac_np    = 12    ! area fraction (of base of mixed (surface) ocean layer)
  INTEGER,PARAMETER::ised_par_top         = 13    ! top height of sedimentary column
  INTEGER,PARAMETER::ised_par_ohmcal      = 14    ! bottom water calcite omega value
  INTEGER,PARAMETER::ised_par_ohmarg      = 15    ! bottom water aragonite omega value
  INTEGER,PARAMETER::ised_par_CO3         = 16    ! bottom water [CO32-]
  INTEGER,PARAMETER::ised_par_Ca          = 17    ! bottom water [Ca2+]
  INTEGER,PARAMETER::ised_par_pH          = 18    ! bottom water pH
  INTEGER,PARAMETER::ised_par_dCO3cal     = 19    ! calcite saturation state ([CO32-] - [CO32-]calsat)
  INTEGER,PARAMETER::ised_par_dCO3arg     = 20    ! aragonite saturation state ([CO32-] - [CO32-]argsat)
  INTEGER,PARAMETER::ised_par_sr_CaCO3    = 21    ! 
  INTEGER,PARAMETER::ised_par_sr_opal     = 22    ! 
  INTEGER,PARAMETER::ised_par_sr_refrac   = 23    ! 
  INTEGER,PARAMETER::ised_par_T           = 24    ! average water temperature over the sedimentary layer
  INTEGER,PARAMETER::ised_par_reminf_Corg = 25    ! value of remineralization function at mid sediment depth
  INTEGER,PARAMETER::ised_par_reminf_opal = 26    ! value of remineralization function at mid sediment depth
  INTEGER,PARAMETER::ised_par_reminf_cal  = 27    ! value of remineralization function at mid sediment depth
  INTEGER,PARAMETER::ised_par_reminf_arg  = 28    ! value of remineralization function at mid sediment depth
  INTEGER,PARAMETER::ised_par_sedcor_f1_cal    = 29 ! user-defined cal sediment core accumulation rate    #1
  INTEGER,PARAMETER::ised_par_sedcor_f1_refrac = 30 ! user-defined refrac sediment core accumulation rate #1
  INTEGER,PARAMETER::ised_par_sedcor_f2_cal    = 31 ! user-defined cal sediment core accumulation rate    #2
  INTEGER,PARAMETER::ised_par_sedcor_f2_refrac = 32 ! user-defined refrac sediment core accumulation rate #2
! sediment system array indices - sediment composition description
  INTEGER,PARAMETER::ised_Corg            = 01    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_FeO             = 02    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_cal             = 03    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_arg             = 04    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_opal            = 05    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_refrac          = 06    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_13C_cal         = 07    ! *** MINIMUM MIXING SET ***
  INTEGER,PARAMETER::ised_13C_arg         = 08    ! *** MINIMUM MIXING SET *** 
  INTEGER,PARAMETER::ised_13C_Corg        = 09    ! 
  INTEGER,PARAMETER::ised_14C_cal         = 10    ! 
  INTEGER,PARAMETER::ised_13C_calfp       = 11    ! planktonic foraminiferal calcite 13C
  INTEGER,PARAMETER::ised_13C_calfb       = 12    ! benthic foraminiferal calcite 13C
  INTEGER,PARAMETER::ised_18O_calfp       = 13    ! planktonic foraminiferal calcite 18O
  INTEGER,PARAMETER::ised_18O_calfb       = 14    ! benthic foraminiferal calcite 18O
  INTEGER,PARAMETER::ised_ash             = 15    ! ash concentration mixing tracer (volume normalized)
  INTEGER,PARAMETER::ised_age_cal         = 16    ! age (tied to calcite sediment fraction)
! surface carbonate chemistry
  INTEGER,PARAMETER::isscc_fCO2           = 01    ! 
  INTEGER,PARAMETER::isscc_cCO2           = 02    ! 
  INTEGER,PARAMETER::isscc_cCO3           = 03    ! 
  INTEGER,PARAMETER::isscc_cHCO3          = 04    ! 
! bathymetry array indices
  INTEGER,PARAMETER::ibath_D              = 01    ! 
  INTEGER,PARAMETER::ibath_Afrac          = 02    ! 
! run-time warnings
  INTEGER,PARAMETER::iwarn_Dcalsat        = 01    ! calsat saturation depth less than zero
  INTEGER,PARAMETER::iwarn_Dargsat        = 02    ! aragonite saturation depth less than zero
  INTEGER,PARAMETER::iwarn_newsedth       = 03    ! thickness of accumulating sediment is > 0.1 cm yr-1
  INTEGER,PARAMETER::iwarn_traceraudit    = 04    ! one of the tracers has drifted beyond acceptable limits
  INTEGER,PARAMETER::iwarn_anoxia         = 05    ! one of the ocean boxes has gone anoxic 

! *** time constants ***
! set the maximum number of simulation years (in kyr)
  INTEGER,PARAMETER::n_kyr       = 500
! set the maximum spin-up time (kry)
  INTEGER,PARAMETER::n_spin      = 500
! Present InterGlacial time (in kyr BP)
  INTEGER,PARAMETER::t_ice_PIG   = 0
  INTEGER,PARAMETER::PIG_t_ice   = 0
! program loop control
  INTEGER,PARAMETER::t_ice_max   = 0
  INTEGER,PARAMETER::t_spin_init = 1
! program loop time step
  INTEGER,PARAMETER::dt_ice      = -1

! *** constants ***
! conversion factors
  REAL(KIND=fp_1),PARAMETER::conv_mol_atm        = 6.024E-21
  REAL(KIND=fp_1),PARAMETER::conv_m3_kg          = 1027.649 ! from Winton and Sarachik [1993] @ 34.7o/oo,0'C
  REAL(KIND=fp_1),PARAMETER::conv_yr_d           = 360.0 !365.25
  REAL(KIND=fp_1),PARAMETER::conv_yr_hr          = 24 * conv_yr_d
  REAL(KIND=fp_1),PARAMETER::conv_yr_s           = 3600.0 * conv_yr_hr
  REAL(KIND=fp_1),PARAMETER::conv_kyr_yr         = 1.0E+03
  REAL(KIND=fp_1),PARAMETER::conv_m_cm           = 1.0E+02
  REAL(KIND=fp_1),PARAMETER::conv_m2_cm2         = 1.0E+04
  REAL(KIND=fp_1),PARAMETER::conv_m3_cm3         = 1.0E+06
  REAL(KIND=fp_1),PARAMETER::conv_kg_g           = 1.0E+03
  REAL(KIND=fp_1),PARAMETER::conv_kg_mg          = 1.0E+06
  REAL(KIND=fp_1),PARAMETER::conv_mol_mmol       = 1.0E+03
  REAL(KIND=fp_1),PARAMETER::conv_mol_umol       = 1.0E+06
  REAL(KIND=fp_1),PARAMETER::conv_mol_nmol       = 1.0E+09
  REAL(KIND=fp_1),PARAMETER::conv_mol_pmol       = 1.0E+12
  REAL(KIND=fp_1),PARAMETER::conv_g_mg           = 1.0E+03
  REAL(KIND=fp_1),PARAMETER::conv_cm3_kg         = conv_m3_kg / conv_m3_cm3
  REAL(KIND=fp_1),PARAMETER::conv_atm_mol        = 1.0 / conv_mol_atm
  REAL(KIND=fp_1),PARAMETER::conv_yr_kyr         = 1.0 / conv_kyr_yr
  REAL(KIND=fp_1),PARAMETER::conv_g_kg           = 1.0 / conv_kg_g
  REAL(KIND=fp_1),PARAMETER::conv_mg_kg          = 1.0 / conv_kg_mg
  REAL(KIND=fp_1),PARAMETER::conv_mmol_mol       = 1.0 / conv_mol_mmol
  REAL(KIND=fp_1),PARAMETER::conv_umol_mol       = 1.0 / conv_mol_umol
  REAL(KIND=fp_1),PARAMETER::conv_nmol_mol       = 1.0 / conv_mol_nmol
  REAL(KIND=fp_1),PARAMETER::conv_pmol_mol       = 1.0 / conv_mol_pmol
  REAL(KIND=fp_1),PARAMETER::conv_cm_m           = 1.0 / conv_m_cm
  REAL(KIND=fp_1),PARAMETER::conv_cm2_m2         = 1.0 / conv_m2_cm2
  REAL(KIND=fp_1),PARAMETER::conv_cm3_m3         = 1.0 / conv_m3_cm3
  REAL(KIND=fp_1),PARAMETER::conv_mg_g           = 1.0 / conv_g_mg
  REAL(KIND=fp_1),PARAMETER::conv_s_yr           = 1.0 / conv_yr_s
  REAL(KIND=fp_1),PARAMETER::conv_kg_cm3         = 1.0 / conv_cm3_kg
  REAL(KIND=fp_1),PARAMETER::conv_d_yr           = 1.0 / conv_yr_d
  REAL(KIND=fp_1),PARAMETER::conv_hr_yr          = 1.0 / conv_yr_hr
! density of calcite
! NOTE: relative molecular mass of calcite is 100.0, density of pure calcite is approximately 2.7
  REAL(KIND=fp_1),PARAMETER::conv_cal_cm3_g      = 2.70
  REAL(KIND=fp_1),PARAMETER::conv_cal_g_cm3      = 1.0 / conv_cal_cm3_g
  REAL(KIND=fp_1),PARAMETER::conv_cal_cm3_mol    = conv_cal_cm3_g / 100.0
  REAL(KIND=fp_1),PARAMETER::conv_cal_mol_cm3    = 1.0 / conv_cal_cm3_mol
! density of aragonite
! NOTE: relative molecular mass of aragonite is 100.0, density of pure aragonite is approximately 2.9
  REAL(KIND=fp_1),PARAMETER::conv_arg_cm3_g      = 2.90
  REAL(KIND=fp_1),PARAMETER::conv_arg_g_cm3      = 1.0 / conv_arg_cm3_g
  REAL(KIND=fp_1),PARAMETER::conv_arg_cm3_mol    = conv_arg_cm3_g / 100.0
  REAL(KIND=fp_1),PARAMETER::conv_arg_mol_cm3    = 1.0 / conv_arg_cm3_mol
! density of opal
! NOTE: relative molecular mass of SiO2 is 60.0, density of opal is in the range 2.0 - 2.5
  REAL(KIND=fp_1),PARAMETER::conv_opal_cm3_g     = 2.25
  REAL(KIND=fp_1),PARAMETER::conv_opal_g_cm3     = 1.0 / conv_opal_cm3_g
  REAL(KIND=fp_1),PARAMETER::conv_opal_cm3_mol   = conv_opal_cm3_g / 60.0
  REAL(KIND=fp_1),PARAMETER::conv_opal_mol_cm3   = 1.0 / conv_opal_cm3_mol
! density of refractory material
! NOTE: assume average density of refractory material (SiO2) as 3.0 (g cm-3)
  REAL(KIND=fp_1),PARAMETER::conv_refrac_cm3_g   = 3.00
  REAL(KIND=fp_1),PARAMETER::conv_refrac_g_cm3   = 1.0 / conv_refrac_cm3_g
  REAL(KIND=fp_1),PARAMETER::conv_refrac_cm3_mol = conv_refrac_cm3_g / 60.0
  REAL(KIND=fp_1),PARAMETER::conv_refrac_mol_cm3 = 1.0 / conv_refrac_cm3_g
! density of iron oxide
! NOTE: the most simple oxide is assumed rather than hematite or magnetite to keep mole calculations simple
! NOTE: relative molecular mass of FeO is 71.8
  REAL(KIND=fp_1),PARAMETER::conv_FeO_cm3_g      = 5.20
  REAL(KIND=fp_1),PARAMETER::conv_FeO_g_cm3      = 1.0 / conv_FeO_cm3_g
  REAL(KIND=fp_1),PARAMETER::conv_FeO_cm3_mol    = conv_FeO_cm3_g / 71.8
  REAL(KIND=fp_1),PARAMETER::conv_FeO_mol_cm3    = 1.0 / conv_FeO_cm3_mol
! density of organic matter
! NOTE: assume average density of particulate organic material as 0.5 (g cm-3)
! NOTE: this is a complete guestimate!!!
  REAL(KIND=fp_1),PARAMETER::conv_Corg_cm3_g     = 0.50
  REAL(KIND=fp_1),PARAMETER::conv_Corg_g_cm3     = 1.0 / conv_Corg_cm3_g    
  REAL(KIND=fp_1),PARAMETER::conv_Corg_cm3_mol   = conv_Corg_cm3_g / 12.0
  REAL(KIND=fp_1),PARAMETER::conv_Corg_mol_cm3   = 1.0 / conv_Corg_cm3_mol
  REAL(KIND=fp_1),PARAMETER::conv_Porg_mol_cm3   = conv_Corg_mol_cm3
  REAL(KIND=fp_1),PARAMETER::conv_Porg_cm3_mol   = conv_Corg_cm3_mol
  REAL(KIND=fp_1),PARAMETER::conv_Norg_cm3_mol   = conv_Corg_cm3_mol
  REAL(KIND=fp_1),PARAMETER::conv_Norg_mol_cm3   = conv_Corg_mol_cm3
  REAL(KIND=fp_1),PARAMETER::conv_Feorg_mol_cm3  = conv_Corg_mol_cm3
  REAL(KIND=fp_1),PARAMETER::conv_Feorg_cm3_mol  = conv_Corg_cm3_mol
! moles of carbon per kg
  REAL(KIND=fp_1),PARAMETER::conv_C_kg_mol       = 83.33
  REAL(KIND=fp_1),PARAMETER::conv_C_mol_kg       = 1.0 / conv_C_kg_mol
  REAL(KIND=fp_1),PARAMETER::conv_CaCO3_mol_KgC  = conv_g_kg * 12.0 
! moles of Fe per kg
  REAL(KIND=fp_1),PARAMETER::conv_Fe_kg_mol      = 17.86
  REAL(KIND=fp_1),PARAMETER::conv_Fe_g_mol       = conv_g_kg * conv_Fe_kg_mol
  REAL(KIND=fp_1),PARAMETER::conv_Fe_mol_kg      = 1.0 / conv_Fe_kg_mol
! moles of SiO2 per kg
  REAL(KIND=fp_1),PARAMETER::conv_SiO2_kg_mol    = 16.67
  REAL(KIND=fp_1),PARAMETER::conv_SiO2_g_mol     = conv_g_kg * conv_SiO2_kg_mol
  REAL(KIND=fp_1),PARAMETER::conv_SiO2_mol_kg    = 1.0 / conv_SiO2_kg_mol
! variable ratios - dependent on [Fe]
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_dia_scale  = 1100000.0  !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_dia_const  = 15000.0    !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_dia_offset = 4.000E-12  ! [Fe']
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_dia_power  = -0.650     !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_coc_scale  = 302581.0   !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_coc_const  = 20000.0    !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_coc_offset = 4.000E-12  ! [Fe']
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_coc_power  = -0.650     !
! variable ratios - dependent on [Fe] (average new production)
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_scale      = 700000.0   !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_const      = 17000.0    !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_offset     = 4.000E-12  ! [Fe']
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_power      = -0.650     !
  REAL(KIND=fp_1),PARAMETER::red_Feorg_Corg_thresh     = 5.765E-12  ! [Fe']
! variable ratios - dependent on [Fe] (Si:N)
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_scale   = 3.4        !
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_const   = 1.0        !
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_offset  = 4.000E-12  ! [Fe']
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_power   = -0.650     !
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_thresh  = 7.525E-12  ! [Fe']
  REAL(KIND=fp_1),PARAMETER::red_Norg_opal_dia_pptonp  = 3.0        ! convert pp to np Si:N ratios
! average continental rock composition
  REAL(KIND=fp_1),PARAMETER::rockav_Si           = 0.205
  REAL(KIND=fp_1),PARAMETER::rockav_Ca           = 0.019
  REAL(KIND=fp_1),PARAMETER::rockav_P            = 0.0007
! Rau et al. [1996,1997] parameter values
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_ed    = 0.7        ! epsilon(d) 13C fractionation factor
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Qsetc = 7.5E-06    ! approximated value of c(i)
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Q2_x2 = +2.829E-10 ! 2nd order polymonial c(i) approximation: x2
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Q2_x  = -1.788E-07 ! 2nd order polymonial c(i) approximation: x
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Q2_c  = +3.170E-05 ! 2nd order polymonial c(i) approximation: c
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Q1_x  = -1.606E-08 ! 1st order polymonial c(i) approximation: x
  REAL(KIND=fp_1),PARAMETER::d13C_DIC_Corg_Q1_c  = +0.834E-05 ! 1st order polymonial c(i) approximation: c
! Wanninkhof [1992] parameter values
  REAL(KIND=fp_1),PARAMETER::pvgas_Sc_CO2_A      = 2.0731E+03
  REAL(KIND=fp_1),PARAMETER::pvgas_Sc_CO2_B      = 1.2562E+02
  REAL(KIND=fp_1),PARAMETER::pvgas_Sc_CO2_C      = 3.6276E+00
  REAL(KIND=fp_1),PARAMETER::pvgas_Sc_CO2_D      = 4.3219E-02
  REAL(KIND=fp_1),PARAMETER::pvgas_k_CO2_a       = 0.5246E+00
  REAL(KIND=fp_1),PARAMETER::pvgas_k_CO2_b       = 1.6256E-02
  REAL(KIND=fp_1),PARAMETER::pvgas_k_CO2_c       = 4.9946E-04
! 13C:(14C+13C+12C) and 13C:12C standards [Bolin, 1981]
  REAL(KIND=fp_1),PARAMETER::standard_13C_C      = 0.011112
  REAL(KIND=fp_1),PARAMETER::standard_13C_12C    = 0.011237
! 14C
  REAL(KIND=fp_1),PARAMETER::standard_14C_12C    = 1.476E-12
! 18O:(18O+17O+16O) [estimated from % natural abundance data]
  REAL(KIND=fp_1),PARAMETER::standard_18O_O      = 0.002004
! lamda for 14C (yrs)
  REAL(KIND=fp_1),PARAMETER::const_lamda_14C        = 5715.000
! yearly fractional reduction factor for 14C ( = EXP[-1.0 / const_lamda_14C] )
  REAL(KIND=fp_1),PARAMETER::const_fracdecay_14C    = 0.999825
! 14C input age fractional reduction factor (400 yrs) ( = EXP[-400.0 / const_lamda_14C] )
  REAL(KIND=fp_1),PARAMETER::const_fracdecay_14C_ip = 0.932402
  
!!$! *** glacial/interglacial physics and tracer constants ***
!!$! set total ocean volume (m3) and area (m2)
!!$  REAL(KIND=fp_1),PARAMETER::PIG_tot_vol         = 1.350E+18
!!$  REAL(KIND=fp_1),PARAMETER::PIG_tot_sur         = 3.500E+14

! *** new primary production constants ***
! growth threshold for phosphate (mol kg-1) - diatom and coccolith growth seperately
  REAL(KIND=fp_1),PARAMETER::bionp_PO4_th_dia    = 0.001E-06
  REAL(KIND=fp_1),PARAMETER::bionp_PO4_th_coc    = 0.001E-06
! growth threshold for iron (mol kg-1) - diatom and coccolith growth seperately
  REAL(KIND=fp_1),PARAMETER::bionp_Fe_th_dia     = 0.001E-09
  REAL(KIND=fp_1),PARAMETER::bionp_Fe_th_coc     = 0.001E-09
! growth threshold for silica (mol kg-1) - diatom and coccolith growth seperately
  REAL(KIND=fp_1),PARAMETER::bionp_Si_th_dia     = 0.100E-06
  REAL(KIND=fp_1),PARAMETER::bionp_Si_th_coc     = 0.100E-06

! *** sediment sub-model constants ***
! dissolution layer thickness (cm)
  REAL(KIND=fp_1),PARAMETER::disslayer_th        = 0.1

! *** look-up table constants ***
! CaCO3 (calcite AND aragonite)
  INTEGER,PARAMETER::lookup_i_D_min      = 0
  INTEGER,PARAMETER::lookup_i_D_max      = 10
  INTEGER,PARAMETER::lookup_i_dCO3_min   = -100
  INTEGER,PARAMETER::lookup_i_dCO3_max   = 100
  INTEGER,PARAMETER::lookup_i_concO2_min = 4    ! (200 umol kg-1)
  INTEGER,PARAMETER::lookup_i_concO2_max = 4    ! (200 umol kg-1)
  INTEGER,PARAMETER::lookup_i_frac_min   = 1
  INTEGER,PARAMETER::lookup_i_frac_max   = 10
  INTEGER,PARAMETER::lookup_i_fCorg_min  = 0
  INTEGER,PARAMETER::lookup_i_fCorg_max  = 50
  REAL(KIND=fp_1),PARAMETER::lookup_D_max      = 10000.0
  REAL(KIND=fp_1),PARAMETER::lookup_dCO3_max   = 100.0 * 1.0E-06 
  REAL(KIND=fp_1),PARAMETER::lookup_concO2_max = 200.0 * 1.0E-06 ! D(concO2) = 50 umol kg-1
  REAL(KIND=fp_1),PARAMETER::lookup_frac_max   = 1.0 
  REAL(KIND=fp_1),PARAMETER::lookup_fCorg_max  = 50.0 * 1.0E-06
! opal
  INTEGER,PARAMETER::lookup_i_opalpc_min       = 1   ! (2%)
  INTEGER,PARAMETER::lookup_i_opalpc_max       = 50  ! (100%)
  INTEGER,PARAMETER::lookup_i_concSi_min       = 0   ! (0 umol kg-1)
  INTEGER,PARAMETER::lookup_i_concSi_max       = 25  ! (250 umol kg-1)
  INTEGER,PARAMETER::lookup_i_T_min            = 270 ! (270 K)
  INTEGER,PARAMETER::lookup_i_T_max            = 280 ! (280 K)
  INTEGER,PARAMETER::lookup_i_KSi0_min         = 1   ! (0.010 yr-1 == 0.0 s-1)
  INTEGER,PARAMETER::lookup_i_KSi0_max         = 100 ! (1.000 yr-1 == 3.16E-08)
  INTEGER,PARAMETER::lookup_i_opaltorefrac_min = 0   ! (0.0 [refrac%/opal%])            
  INTEGER,PARAMETER::lookup_i_opaltorefrac_max = 10  ! (10.0 [refrac%/opal%]) 
  REAL(KIND=fp_1),PARAMETER::lookup_opalpc_max       = 1.0               ! D(opalpc) = 2%
  REAL(KIND=fp_1),PARAMETER::lookup_concSi_max       = 250.0 * 1.0E-06   ! D(concSi) = 10 umol kg-1
  REAL(KIND=fp_1),PARAMETER::lookup_T_max            = 280.0             ! D(T)      = 1 K
  REAL(KIND=fp_1),PARAMETER::lookup_KSi0_max         = 1.000 / conv_yr_s ! D(KSi0)   = 0.010 yr-1
  REAL(KIND=fp_1),PARAMETER::lookup_opaltorefrac_max = 10.0              ! D(opaltorefrac) = 1.0

! *** misc *** 
! value of yearly time-step
  REAL(KIND=fp_1),PARAMETER::dt_y                = 1.0
! zero degree centigrade in Kelvin
  REAL(KIND=fp_1),PARAMETER::const_zeroC         = 273.15
! gas constant R ( = 83.145 bar cm3 mol-1 K-1)
  REAL(KIND=fp_1),PARAMETER::const_R             = 83.145
! PI
  REAL(KIND=fp_1),PARAMETER::const_PI            = 3.141593
! Radius of the Earth (m)
  REAL(KIND=fp_1),PARAMETER::const_rEarth        = 6.371E+06
! assumed freezing point of seawater (K)
  REAL(KIND=fp_1),PARAMETER::const_freezesea     = const_zeroC - 1.9
! zero value REAL constant
  REAL(KIND=fp_1),PARAMETER::dummy_constant      = 0.0
! integer value for the error log file pipe unit number
  INTEGER,PARAMETER::errorlog_unitnum            = 1
! "yes" and "no" (!)
  CHARACTER(LEN=1),PARAMETER::yes = "y"
  CHARACTER(LEN=1),PARAMETER::no  = "n"
! misc constants
  INTEGER,PARAMETER::const_integer_1             = 1
  REAL(KIND=fp_1),PARAMETER::const_real_1        = REAL(const_integer_1,KIND=fp_1)

! *************************************************************************************************



! *************************************************************************************************
! *** GLOBAL CUSTOM DATA-TYPES ********************************************************************
! *************************************************************************************************
  

! ocean circulation description
  TYPE ocean_circulation
    INTEGER::ioc_cs           ! start column
    INTEGER::ioc_ls           ! start layer
    INTEGER::ioc_ce           ! end column
    INTEGER::ioc_le           ! end layer
    REAL(KIND=fp_1)::ioc_wf   ! current ocean circulation water flux
    REAL(KIND=fp_1)::ioc_wf_1 ! ocean circulation water flux scheme #1
    REAL(KIND=fp_1)::ioc_wf_2 ! ocean circulation water flux scheme #2
  END TYPE ocean_circulation

! derivative function
! NOTE: for the np fluxes to the user-defined sediment cores,
!       because the number of cores is not set until later, 
!       and this type of construction does not appear to be able to take an 'ALLOCATABLE' statement, 
!       a number for the maximum allowable number of cores is used to define the array size
  TYPE derivative_function
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_tracer)::idf_t        ! ocean tracers
    REAL(KIND=fp_1),DIMENSION(n_atm)::idf_a                       ! atmospheric tracers
    REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::idf_s ! new production sediment flux (to sed)
    REAL(KIND=fp_1),DIMENSION(n_sedcor_max,n_sed_fnp)::idf_c      ! new production sediment flux (to sedcor)
    REAL(KIND=fp_1),DIMENSION(n_col,n_bionp)::idf_b               ! ocean biology: total new production
    REAL(KIND=fp_1),DIMENSION(n_tracer)::idf_g                    ! "dtracer_tot" tracer audit
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_t_int_max)::idf_v     ! monthly convective adjustments
  END TYPE derivative_function

! combined ocean-atmosphere tracer description
  TYPE ocean_atm_tracer
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_tracer)::ioat_t       ! 
    REAL(KIND=fp_1),DIMENSION(n_atm)::ioat_a                      ! 
  END TYPE ocean_atm_tracer
  
! *************************************************************************************************


  
! *************************************************************************************************
! *** GLOBAL VARIABLES ****************************************************************************
! *************************************************************************************************


! *** Redfield Ratios ***
! primary (total new production)
  REAL(KIND=fp_1)::red_Porg_Corg  = 0.0 !106.0
  REAL(KIND=fp_1)::red_Porg_Norg  = 0.0 !16.0
  REAL(KIND=fp_1)::red_Porg_O2    = 0.0 !177.0
  REAL(KIND=fp_1)::red_Corg_Porg  = 0.0 !1.0 / red_Porg_Corg 
  REAL(KIND=fp_1)::red_Corg_O2    = 0.0 !red_Porg_O2 / red_Porg_Corg
  REAL(KIND=fp_1)::red_Corg_Norg  = 0.0 !red_Porg_Norg / red_Porg_Corg
! used-set new production Redfield Ratios (silicate- and carbonate-shell based new production seperately)
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Feorg_Corg_dia  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Feorg_Corg_coc  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_opal_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_cal_coc    = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_arg_coc    = 0.0
! derived new production Redfield Ratios (silicate- and carbonate-shell based new production seperately)
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Corg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Corg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Porg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Porg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Norg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Norg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Norg_Porg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Norg_Porg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_opal_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_opal_Porg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_opal_Norg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_cal_Corg_coc    = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_arg_Corg_coc    = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_opal_Corg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Norg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Norg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Norg_Corg_dia   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Norg_Corg_coc   = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Feorg_dia  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Corg_Feorg_coc  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Feorg_dia  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Porg_Feorg_coc  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Feorg_Porg_dia  = 0.0
  REAL(KIND=fp_1),DIMENSION(n_col)::red_Feorg_Porg_coc  = 0.0
  

  
! *************************************************************************************************
! *** GLOBAL PARAMETERS ***************************************************************************
! *************************************************************************************************

! *** MODEL CONFIGURATION: INTEGRATION CONTROL ***
! numerical integration scheme
  INTEGER::par_int_scheme
! number of mode time steps per year
  INTEGER::par_n_t_int
! number of sub-steps per new production calculation time-step
  INTEGER::par_n_t_np
! simulation time length definition
  INTEGER::t_ice_init  = 0
  INTEGER::t_spin_max  = 0
  INTEGER::t_ice_load  = 0
  INTEGER::t_ice_save  = 0
  INTEGER::t_ice_chain = 0
! results print-out interval
  INTEGER::t_print = 0
! interval (years) between each ocean volume adjustment
  INTEGER::par_t_dt
! timeslices for data saving
  INTEGER::par_t_ice_save_1
  INTEGER::par_t_ice_save_2

! *** sediments: misc ***
! sediment interface dissolution layer thickness (cm)
  REAL(KIND=fp_1)::par_sed_interf_th
! depth of bioturbated layer below top ('well-mixed') layer (integer number of cm),
! i.e., the number of 1 cm sediment sub-layers to be mixed
  INTEGER::n_sed_mix
! maximum bioturbation profile mixing rate (cm2 kyr-1), i.e., corresponding to unity in the profile data file
  REAL(KIND=fp_1)::par_sed_mix_kmax
! frction of carbonate dissolution that proceeds by 'interface' rather than 'homogeneous' dissolution
  REAL(KIND=fp_1)::par_sed_fraci
! carbonate dissolution "rate constant"
  REAL(KIND=fp_1)::par_caldis_k
  REAL(KIND=fp_1)::par_argdis_k
! carbonate dissolution exponent
  REAL(KIND=fp_1)::par_caldis_exp
  REAL(KIND=fp_1)::par_argdis_exp
! flux of detrital material
  REAL(KIND=fp_1)::par_frefrac
! initial composition of old and mixed-layer sediments
  REAL(KIND=fp_1)::par_init_sed_cal
  REAL(KIND=fp_1)::par_init_sed_arg
  REAL(KIND=fp_1)::par_init_sed_opal
  REAL(KIND=fp_1)::par_init_sed_Corg
! initial sediment CaCO3 concentration (cm3(solid) / cm3(sed))
  REAL(KIND=fp_1)::par_sed_CaCO3_init
! base opal dissolution rate constant (intercept at zero opal rain rate)
  REAL(KIND=fp_1)::par_sed_opal_KSi0
! asymptotic [Si] %refrac/%opal ratio max limit
  REAL(KIND=fp_1)::par_sed_opal_Sitoopalmax
! fraction of sedimentating POM flux preserved in the sediments
  REAL(KIND=fp_1)::par_sed_presfrac_Corg
! fraction of sedimentating scavenged Fe and POM Fe flux preserved in the sediments
  REAL(KIND=fp_1)::par_sed_presfrac_FeO
! fraction of Corg rain that is available for driving diagenetic CaCO3 dissolution
  REAL(KIND=fp_1)::par_sed_diagenfrac_Corg

! *** sediments: sub-system ***
! number of sediment compositional elements to mix
  INTEGER::n_sed_com_min
! top ('well-mixed') sediment layer thickness (cm)
  REAL(KIND=fp_1)::par_sed_top_th
! sediment porosity (cm3(pore water) / cm3(sed))
  REAL(KIND=fp_1)::par_sed_poros
! sediment porosity in top layer (cm3(pore water) / cm3(sed))
! NOTE: this value is averaged over the depth of the top (well mixed) sediment layer
  REAL(KIND=fp_1)::par_sed_poros_top
  
! *** sediments: used-definable cores ***
! number of sediment compositional elements to mix
  INTEGER::n_sedcor_com_min
! number of sediment cores
  INTEGER::n_sedcor
! number of sedimentary core stack sub-layers
  INTEGER::n_sedcor_tot
! top ('well-mixed') sediment layer thickness (cm)
  REAL(KIND=fp_1)::par_sedcor_top_th
! sediment porosity (cm3(pore water) / cm3(sed))
  REAL(KIND=fp_1)::par_sedcor_poros
! sediment porosity in top layer (cm3(pore water) / cm3(sed))
! NOTE: this value is averaged over the depth of the top (well mixed) sediment layer
  REAL(KIND=fp_1)::par_sedcor_poros_top

! *** sediments: data save ***
!  estimated lysocline threshold (%)
  REAL(KIND=fp_1)::par_sed_lys_thresh

! *** new production: misc ***
! temperature-dependent "Q10" kinetic constant
  REAL(KIND=fp_1)::par_bionp_kT_Q10_dia
  REAL(KIND=fp_1)::par_bionp_kT_Q10_coc
! temperature-dependent "a1" kinetic constant
  REAL(KIND=fp_1)::par_bionp_kT_a1_dia
  REAL(KIND=fp_1)::par_bionp_kT_a1_coc
! species-specific [PO4]-uptake kinetic factor (at 0'C and no light limitation)
  REAL(KIND=fp_1)::par_bionp_kmax_dia
  REAL(KIND=fp_1)::par_bionp_kmax_coc
! species-specific [PO4]-uptake rate (mol kg-1 yr-1)
  REAL(KIND=fp_1)::par_bionp_dSSPO4_dia
  REAL(KIND=fp_1)::par_bionp_dSSPO4_coc
! fraction of CaCO3 new production as aragnoite
  REAL(KIND=fp_1)::par_bio_frac_arg
! half-saturation Michaelis-Menten value for phosphate (mol kg-1) - diatom and coccolith growth seperately
  REAL(KIND=fp_1)::par_bionp_PO4_0_dia
  REAL(KIND=fp_1)::par_bionp_PO4_0_coc
! half-saturation Michaelis-Menten value for iron (mol kg-1) - diatom and coccolith growth seperately
! NOTE: as dissolved inorganic iron ([Fe'])
  REAL(KIND=fp_1)::par_bionp_Feprime_0_dia
  REAL(KIND=fp_1)::par_bionp_Feprime_0_coc
! half-saturation Michaelis-Menten value for iron (mol kg-1) - diatom and coccolith growth seperately
! NOTE: as dissolved inorganic iron ([Fe])
  REAL(KIND=fp_1)::par_bionp_Fe_0_dia
  REAL(KIND=fp_1)::par_bionp_Fe_0_coc
! half-saturation Michaelis-Menten value for silica (mol kg-1) - diatom and coccolith growth seperately
  REAL(KIND=fp_1)::par_bionp_Si_0_dia
  REAL(KIND=fp_1)::par_bionp_Si_0_coc
! fraction of diatom organic matter new production in DOM form
  REAL(KIND=fp_1)::par_bionp_frac_DOM_dia
! fraction of coccolithophore organic matter new production in DOM form
  REAL(KIND=fp_1)::par_bionp_frac_DOM_coc
! DOM e-folding decay time (yr-1)
  REAL(KIND=fp_1)::par_bionp_DOM_tconst
! restoring time constant for SSPO4 (yr-1)
  REAL(KIND=fp_1)::par_misc_SSPO4_tconst
! np kinetic factor - deep surface convective mixing
  REAL(KIND=fp_1)::par_bionp_kconv

! *** new production: Redfield Ratios ***
! primary ratios
  REAL(KIND=fp_1)::par_red_Porg_Corg
  REAL(KIND=fp_1)::par_red_Porg_Norg
  REAL(KIND=fp_1)::par_red_Porg_O2
! fixed ratios
  REAL(KIND=fp_1)::par_red_Feorg_Corg_dia
  REAL(KIND=fp_1)::par_red_Feorg_Corg_coc
  REAL(KIND=fp_1)::par_red_Corg_opal_dia 
  REAL(KIND=fp_1)::par_red_Corg_cal_coc
  REAL(KIND=fp_1)::par_red_Corg_arg_coc
! maximum diatom Fe:C ratio
  REAL(KIND=fp_1)::par_red_Feorg_Corg_dia_max
! maximum coccolithophore Fe:C ratio
  REAL(KIND=fp_1)::par_red_Feorg_Corg_coc_max
! value for variable diatom C:Si ratio : Fe-deplete
  REAL(KIND=fp_1)::par_red_Corg_opal_dia_var_r0
! value for variable diatom C:Si ratio : [Fe] offset
  REAL(KIND=fp_1)::par_red_Corg_opal_dia_var_dFe
! value for variable diatom C:Si ratio : [Fe] half saturation value
  REAL(KIND=fp_1)::par_red_Corg_opal_dia_var_Fe0

! *** new production: remineralization ***
! remineralization e-folding length scales for Corg, aragonite and opal (m)
  REAL(KIND=fp_1)::par_remin_eL_Corg
  REAL(KIND=fp_1)::par_remin_eL_cal
  REAL(KIND=fp_1)::par_remin_eL_arg
  REAL(KIND=fp_1)::par_remin_eL_opal
! remineralization power function characteristic length
  REAL(KIND=fp_1)::par_remin_pL_Corg
! remineralization power function characteristic power value
  REAL(KIND=fp_1)::par_remin_pP_Corg
! recalitrant Corg fraction
  REAL(KIND=fp_1)::par_remin_fracrecal_Corg
! recalitrant cal fraction
  REAL(KIND=fp_1)::par_remin_fracrecal_cal
! reference remineralization base layer number
  INTEGER::par_remin_nlbase
! minimum allowable dissolved oxygen concentration
  REAL(KIND=fp_1)::par_remin_O2_min
! opal water-column remineralization settling velocity (m d-1)
  REAL(KIND=fp_1)::par_remin_opal_vel
! opal water-column remineralization - dissolution rate constant (0'C, completely undersaturated) (d-1)
  REAL(KIND=fp_1)::par_remin_opal_K

! *** new production: isotopic fractionation ***
! fixed DIC:Corg d13C fractionation value for diatoms
  REAL(KIND=fp_1)::par_bionp_13C_DIC_Corg_dia
! fixed DIC:Corg d13C fractionation value for coccolithophores
  REAL(KIND=fp_1)::par_bionp_13C_DIC_Corg_coc
! epsilon(f) value for diatoms
  REAL(KIND=fp_1)::par_bionp_13C_DIC_Corg_ef_dia
! epsilon(f) value for coccolithophores
  REAL(KIND=fp_1)::par_bionp_13C_DIC_Corg_ef_coc

! *** new production: fixed magnitude ***

! *** Iron! ***
! fraction of iron in (nominally refractory) terrigeneous matter (by weight)
  REAL(KIND=fp_1)::par_frac_dust_Fe
! fraction of Fe which dissolves
  REAL(KIND=fp_1)::par_frac_Fe_dis
! 'apparent' solubility of iron (after Johnson et al. [1997])
  REAL(KIND=fp_1)::par_apparsol_Fe
! scavenging rate constant for iron (after Johnson et al. [1997])
  REAL(KIND=fp_1)::par_k_scav_Fe
! scavenging rate constant proportionality to particulate matter (PM) flux ((mol m-2 yr-1)-1) 
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_Corg
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_opal
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_cal
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_arg
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_refrac
  REAL(KIND=fp_1)::par_k_scav_Fe_fPMlaw_tot
! mixed layer scavenging rate multiplication factor
  REAL(KIND=fp_1)::par_k_scav_Fe_sur_mult
! fudge factor to convert [Fe(total)] to [Fe']
  REAL(KIND=fp_1)::par_conv_Fe_tot_inorg

! *** GAIA SUB-SYSTEM: MISC ***
! fraction of SiO2 in (nominally refractory) terrigeneous matter (by weight)
  REAL(KIND=fp_1)::par_frac_dust_SiO2
! fraction of SiO2 which dissolves
  REAL(KIND=fp_1)::par_frac_SiO2_dis
! riverine inputs
  REAL(KIND=fp_1),DIMENSION(n_tracer)::par_gaia_riv
! volcanic (including terrestrial outgassing and hydrothermal) inputs
  REAL(KIND=fp_1),DIMENSION(n_tracer)::par_gaia_vol
! miscellaneous inputs
  REAL(KIND=fp_1),DIMENSION(n_tracer)::par_gaia_misc
! proportion of riverine flux added to the surface ocean only (rather than to the whole ocean)
  REAL(KIND=fp_1)::par_gaia_frac_riv_sur
! proportion of volcanic flux added to the surface ocean only (rather than to the whole ocean)
  REAL(KIND=fp_1)::par_gaia_frac_vol_sur
! proportion of miscellaneous flux added to the surface ocean only (rather than to the whole ocean)
  REAL(KIND=fp_1)::par_gaia_frac_misc_sur
! fraction of terrestrial carbon transferred to atmospheric box rather than to the ocean,
! and proportion of the (remaining) flux added to the surface ocean only (rather than to the whole ocean)
  REAL(KIND=fp_1)::par_gaia_frac_terr_atm
  REAL(KIND=fp_1)::par_gaia_frac_terr_sur

! *** GAIA SUB-SYSTEM: ISOTOPIC SIGNITURES ***
! NOTE: the delta 13C value of '1.8' is approximately equal to the average d13C of buried CaCO3 in the model
! volcanic CO2 13C signature
  REAL(KIND=fp_1)::par_gaia_13C_vol  != +2.2
! riverine 13C signature
  REAL(KIND=fp_1)::par_gaia_13C_riv  != +2.2
! coral reef CaCO3 13C signature
  REAL(KIND=fp_1)::par_gaia_13C_neri != +2.2
! misc 13C signature
  REAL(KIND=fp_1)::par_gaia_13C_misc != +2.2
! terrestrial vegetation CO2 13C signature
  REAL(KIND=fp_1)::par_gaia_13C_terr != -25.0
  
! *** GAIA SUB-SYSTEM: NERITIC SEDIMENTS STORAGE ***
! fraction of reef CaCO3 transferred to/from the surface ocean only (rather than to the whole ocean)
  REAL(KIND=fp_1)::par_gaia_frac_neri_sur
! reefal CaCO3 accumulation rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_reefal_racc
! reefal CaCO3 errosion rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_reefal_rero
! max water depth for reefal CacO3 accumulation (m)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_reefal_maxD
! non-reefal CaCO3 accumulation rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_nonreefal_racc
! non-reefal CaCO3 errosion rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_nonreefal_rero
! max water depth for non-reefal CacO3 accumulation (m)
  REAL(KIND=fp_1)::par_gaia_neri_CaCO3_nonreefal_maxD
! neritic opal accumulation rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_opal_racc
! neritic opal errosion rate (mol m-2 kyr-1)
  REAL(KIND=fp_1)::par_gaia_neri_opal_rero
! max water depth for neritic opal accumulation (m)
  REAL(KIND=fp_1)::par_gaia_neri_opal_maxD
! maximum sediment depth (below PIG sea level) to be considered for neritic sediment accumulation
! NOTE: these value is not entered directly from the parameter file, but rather is calculated
  INTEGER::par_n_bath_depth_neri

! *** MISC: MISC ***
! number of water mass flux vectors (max 9999)
  INTEGER::par_n_wflux
! (fixed) CO2 ocean-atmosphere gas transfer coefficient (mol m-2 yr-1 uatm-1)
  REAL(KIND=fp_1)::par_misc_kgastrans_coeff_CO2
! O2 ocean-atmosphere gas transfer velocity (m yr-1)
  REAL(KIND=fp_1)::par_misc_kgastrans_vel_O2
! vertical eddy diffusivity
  REAL(KIND=fp_1)::par_misc_vdif
! horizontal eddy diffusivity
  REAL(KIND=fp_1)::par_misc_hdif
! maximum potential fractional convective adjustment (yr-1)
  REAL(KIND=fp_1)::par_misc_conv_adj_max
! (normalized) insolation threshold for convective adjustment at surface
! NOTE: at insolation levels above this value, no surface convection is allowed
  REAL(KIND=fp_1)::par_misc_conv_Ithresh
! temperature adjustment for water convected down from the surface
! NOTE: this is to account for the fact that although only annual SSTs are available,
!       convection (and thus injection of surface water to depth) takes place 
!       during the winter months when SSTs are lower than the annula mean
  REAL(KIND=fp_1)::par_misc_conv_dT
! minimum convected temperature
  REAL(KIND=fp_1)::par_misc_conv_Tmin
! salinity adjustment for water convected down from the surface
! NOTE: this is to account for the fact that although only annual SSSs are available,
!       convection (and thus injection of surface water to depth) takes place 
!       during the winter months when SSSs are higher than the annual mean
  REAL(KIND=fp_1)::par_misc_conv_dS
! minimum convected temperature
  REAL(KIND=fp_1)::par_misc_conv_Smax
! restoring time constant for SSTs and SSSs (yr-1)
  REAL(KIND=fp_1)::par_misc_SS_tconst
! restoring time constant for SSO2 (yr-1)
  REAL(KIND=fp_1)::par_misc_SSO2_tconst

! *** DEBUG ***
! threshold for mass fraction warning
  REAL(KIND=fp_1)::par_boxfracmassex_threshold
! threshold value for tracer inventory drift (%)
  REAL(KIND=fp_1)::par_dtot_per_threshold


! *************************************************************************************************



! *************************************************************************************************
! *** GLOBAL VARIABLES ****************************************************************************
! *************************************************************************************************


! *** temporary variables ***
  CHARACTER(LEN=100)::dummy_string = ""
  REAL(KIND=fp_1)::dummy_real = 0.0
  INTEGER::dummy_integer = 0
  INTEGER::error,alloc_error,dealloc_error
  CHARACTER(LEN=1)::program_error = no

! *** initialize bioturbation mixing rate array ***
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:)::sed_mix_k             ! bioturbation mixing rate profile array

! *** initialize integral sediment stacks ***
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:,:)::sed_save        ! hold reordered data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::sed_save_age_tot  ! sediment age data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::sed_save_age_ash  ! sediment age data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::sed_save_age_14C ! sediment age data saving array

! *** initialize user-definable sediment stacks ***
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::sedcor            ! 
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_top          ! 
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_par          ! 
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_disconst     ! 
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_tracer       ! 
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:)::sedcor_save       ! hold reordered data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_save_age_tot ! sediment age data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_save_age_ash ! sediment age data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_save_age_14C ! sediment age data saving array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_fnp          !
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:)::sedcor_fdis         !

! *** look-up tables ***
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:,:)::lookup_sed_dis_cal
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:,:)::lookup_sed_dis_arg
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:,:,:,:,:)::lookup_sed_dis_opal

! *** np time step (ocean surface) cycle arrays ***
  REAL(KIND=fp_1),DIMENSION(n_t_np_max,n_col)::seas_t_np_insol        ! insolation cycle
  REAL(KIND=fp_1),DIMENSION(n_t_np_max,n_col)::seas_t_np_seaice       ! seaice cycle
  REAL(KIND=fp_1),DIMENSION(n_t_np_max,n_col,n_tracer)::seas_t_np_dSS ! SS tracer
  REAL(KIND=fp_1),DIMENSION(n_t_np_max,n_col,n_bionp)::seas_t_np_dnp  ! np
  REAL(KIND=fp_1),DIMENSION(n_t_np_max,n_col,n_biok)::seas_t_np_dk    ! kinetics

! *** seasonal time step cycle arrays ***
  REAL(KIND=fp_1),DIMENSION(n_t_int_max,n_col)::seas_t_int_insol         ! insolation cycle
  REAL(KIND=fp_1),DIMENSION(n_t_int_max,n_col)::seas_t_int_seaice        ! seaice cycle
  TYPE(ocean_atm_tracer),DIMENSION(n_t_int_max)::seas_t_int_oatracer     ! SS tracer
  REAL(KIND=fp_1),DIMENSION(n_t_int_max,n_col,n_bionp)::seas_t_int_bionp ! np

! *** misc ***
! run-time warnings
! NOTE: initialize all values in array to 'no' (+0)
  CHARACTER(LEN=1),DIMENSION(10)::warning = no
! dummy array
  REAL(KIND=fp_1),ALLOCATABLE,DIMENSION(:)::dummy_array_REAL

! *************************************************************************************************


CONTAINS


! *************************************************************************************************
! *** ISOTOPE CALCULATION ROUTINES ****************************************************************
! *************************************************************************************************

! *** calculate d13C (from mol 13C and mol C) ***
  FUNCTION calc_d13C(totC,tot13C)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_d13C
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::totC,tot13C
! return function value
! NOTE: trap zero input values
    IF (totC > 0.0) THEN
      calc_d13C = 1000.0 * ((tot13C / totC) - standard_13C_C) / standard_13C_C
    ELSE
      calc_d13C = 0.0
    ENDIF
! filter out possible 'abnormal' values
    IF (calc_d13C > 99.999)  calc_d13C = 99.999
    IF (calc_d13C < -99.999) calc_d13C = -99.999
  END FUNCTION calc_d13C
  
  
! *** calculate fraction of 13C in total carbon ***
  FUNCTION calc_frac_13C(d13C)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_frac_13C
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::d13C
! return function value
    calc_frac_13C = standard_13C_C * (1 + d13C / 1000.0)
  END FUNCTION calc_frac_13C
  
  
! *** calculate d13C fractionation during a process ***
  FUNCTION calc_d13Cfrac(d13C)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_d13Cfrac
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::d13C
! return function value
    calc_d13Cfrac = standard_13C_C * (d13C / 1000.0)
  END FUNCTION calc_d13Cfrac
  
  
! *** calculate d18O (from mol 18O and mol O) ***
  FUNCTION calc_d18O(totO,tot18O)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_d18O
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::totO,tot18O
! return function value
! NOTE: trap zero input values
    IF (totO > 0.0) THEN
      calc_d18O = 1000.0 * ((tot18O / totO) - standard_18O_O) / standard_18O_O
    ELSE
      calc_d18O = 0.0
    ENDIF
! filter out possible 'abnormal' values
    IF (calc_d18O > 99.999)  calc_d18O = 99.999
    IF (calc_d18O < -99.999) calc_d18O = -99.999
  END FUNCTION calc_d18O
  
  
! *** CALCULATE FRACTION OF 18O IN TOTAL OXYGEN ***
  FUNCTION calc_frac_18O(d18O)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_frac_18O
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::d18O
! return function value
    calc_frac_18O = standard_18O_O * (1 + d18O / 1000.0)
  END FUNCTION calc_frac_18O
  

! *** calculate d18O fractionation during a process ***
  FUNCTION calc_d18Ofrac(d18O)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_d18Ofrac
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::d18O
! return function value
    calc_d18Ofrac = standard_18O_O * (d18O / 1000.0)
  END FUNCTION calc_d18Ofrac
  
  

! *************************************************************************************************



! *************************************************************************************************
! *** MISC ROUTINES *******************************************************************************
! *************************************************************************************************
  
  
! *** calculate the density of sea water ***
! NOTE: from Winton and Sarachik [1993]
! NOTE: rho in units of (kg m-3)
! NOTE: salinity is in (o/oo)
! NOTE: temperature must be converted from (K) to (degrees Celcius)
  FUNCTION calc_rho(T,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_rho
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::T
    REAL(KIND=fp_1),INTENT(in)::S
! local variables
    REAL(KIND=fp_1)::T_C
! convert units of local variables
    T_C = T - const_zeroC
! return function value
    calc_rho = 1000.0 + (0.7968 * S - 0.0559 * T_C - 0.0063 * T_C**2 + 3.7315E-05 * T_C**3)
  END FUNCTION calc_rho


! *** write to the error log file ***
  SUBROUTINE log_write(writeopt,text,num)
! dummy arguments
    INTEGER,INTENT(in)::writeopt
    CHARACTER(LEN=*),INTENT(in)::text
    REAL(KIND=fp_1),INTENT(in)::num
! local variables
    INTEGER::out=errorlog_unitnum
! write to file
    SELECT CASE (writeopt)
    CASE (1) !text only
      WRITE(unit=out,FMT='(A100)') text
    CASE (2) !text followed by number
      WRITE(unit=out,FMT='(A100,E13.6)') text
    END SELECT
  END SUBROUTINE log_write


! *** quadratic equation solving routine ***
  FUNCTION quad_root(a,b,c)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(2)::quad_root
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::a,b,c
! return value
    quad_root(1) = (-b + (b**2 - 4 * a * c)**0.5) / (2 * a)
    quad_root(2) = (-b - (b**2 - 4 * a * c)**0.5) / (2 * a)
  END FUNCTION quad_root


! *** convert box number into character string ***
  FUNCTION conv_num_char(n)
    IMPLICIT NONE
! result variable
    CHARACTER(LEN=2)::conv_num_char
! dummy valiables
    INTEGER,INTENT(in)::n
! return value
    IF (n<=9) THEN
      WRITE(conv_num_char(2:2),'(i1)')n
      conv_num_char(1:1) = "0"
    ELSE
      WRITE(conv_num_char(1:2),'(i2)')n
    ENDIF
  END FUNCTION conv_num_char


! *****************************
! *** save kyr data to file ***
! *****************************
! NOTE: this is a general data-saving routine for complete glacial/interglacial run data-sets
  SUBROUTINE save_kyrdata(sdata,filename,mult)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_kyr)::sdata
    CHARACTER(LEN=*),INTENT(in)::filename
    REAL,INTENT(in)::mult
! local variables
    INTEGER::t=0,out=1
! open file pipe
    OPEN(unit=out,file=filename,action="write")
! save the data
    DO t = 0,n_kyr
      WRITE(unit=out,FMT='(I6,F8.3)') t,mult*sdata(t)
    END DO
    CLOSE(unit=out)
  END SUBROUTINE save_kyrdata
! *****************************
  
  
! ******************************
! *** check data file format ***
! ******************************
  SUBROUTINE check_fileformat(ndata,filename)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::ndata
    CHARACTER(LEN=*),INTENT(in)::filename
! local variables
    INTEGER::n
    INTEGER::in=5
    CHARACTER(LEN=27)::char_input
    
! *** check that the number of lines of data in the file is correct ***
! NOTE: the position of the "------- END OF DATA -------" marker is checked
! open file pipe
    OPEN(unit=in,file=filename,action="read")
! read in data
    DO n = 1,10000
      READ(unit=in,FMT='(A27)') char_input
      IF (char_input == "------- END OF DATA -------") THEN
	IF (n == (ndata + 1)) THEN
	  EXIT
	ELSE
	  PRINT*,""
	  PRINT*,"FATAL ERROR:"
	  PRINT*,"too few data elements in input data file"
	  PRINT*,filename
	  PRINT*,""
	  STOP
	ENDIF
      ELSE
	IF (n == (ndata + 1)) THEN
	  PRINT*,""
	  PRINT*,"FATAL ERROR:"
	  PRINT*,"missing '------- END OF DATA -------' tag or"
	  PRINT*,"too many data elements in input data file"
	  PRINT*,filename
	  PRINT*,""
	  STOP
	ENDIF
      ENDIF
    END DO
! close file pipe
    CLOSE(unit=in)
    
  END SUBROUTINE check_fileformat
! ******************************


! *************************************************************
! *** calculate sediment/ocean layer fractional adjacencies ***
! *************************************************************
  FUNCTION calc_sed_lay_frac(sedpar,phys)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(n_lay)::calc_sed_lay_frac
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_lay,n_phys)::phys
! local variables
    INTEGER::l
    INTEGER::lay_bot,lay_top,lay_bot_top_num
    REAL(KIND=fp_1)::sumfrac
! calculate number of adjacted ocean layers to the sediment layer
    lay_bot = INT(sedpar(ised_par_lay_bot))
    lay_top = INT(sedpar(ised_par_lay_top))
    lay_bot_top_num = lay_bot - lay_top + 1
! zero function value
    calc_sed_lay_frac(:) = 0.0
    
! *** estimate fractional connectivity of ocean layers to sediment layer and return function value***
    SELECT CASE (lay_bot_top_num)
    CASE (0)
      PRINT*,"FATAL ERROR [sue_lib.f90, calc_sed_lay_frac]:"
      PRINT*,"there are apparently no ocean layers adjacent to the current sediment layer!"
      STOP
    CASE (1)
      calc_sed_lay_frac(lay_top) = 1.0
    CASE (2)
      calc_sed_lay_frac(lay_top) = (phys(lay_top,ip_D_bot) - sedpar(ised_par_D_top)) / &
	& (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
      calc_sed_lay_frac(lay_bot) = (sedpar(ised_par_D_bot) - phys(lay_bot,ip_D_top)) / &
	& (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
    CASE default
      calc_sed_lay_frac(lay_top) = (phys(lay_top,ip_D_bot) - sedpar(ised_par_D_top)) / &
	& (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
      DO l = (lay_top + 1),(lay_bot - 1),-1
	calc_sed_lay_frac(l) = (phys(l,ip_D_bot) - phys(l,ip_D_top)) / &
	  & (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
      ENDDO
      calc_sed_lay_frac(lay_bot) = (sedpar(ised_par_D_bot) - phys(lay_bot,ip_D_top)) / &
	& (sedpar(ised_par_D_bot) - sedpar(ised_par_D_top))
    END SELECT

! *** check that fraction transfer vector array sums to zero!
    sumfrac = SUM(calc_sed_lay_frac(:))
    IF (sumfrac < 0.999999 .OR. sumfrac > 1.000001) THEN
      PRINT*,"FATAL ERROR [sue_lib.f90, calc_sed_lay_frac]:"
      PRINT*,"SUM ( sed_lay_frac ) = ",sumfrac
      STOP
    ENDIF
    
  END FUNCTION calc_sed_lay_frac
! *************************************************************

  
! ****************************************************************
! *** calculate ocean layer adjacent to the mid sediment depth ***
! ****************************************************************
  FUNCTION calc_sed_lay_mid(sedmidD,phys)
    IMPLICIT NONE
! result variable
    INTEGER::calc_sed_lay_mid
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::sedmidD
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_lay,n_phys)::phys
! local variables
    INTEGER::l
    INTEGER::lay_mid
    
! *** calculate number of the ocean layer adjacent to the mid sediment depth and return function value***
    DO l = 1,n_lay
      IF (phys(l,ip_D_bot) >= sedmidD) THEN
	calc_sed_lay_mid = l
	EXIT
      ENDIF
    END DO
    
  END FUNCTION calc_sed_lay_mid
! ****************************************************************


! **********************************************************
! *** general linear 4 dimensional interpolation routine ***
! **********************************************************
  FUNCTION interp_4D(array,a,b,c,d,a_max,b_max,c_max,d_max, &
    & i_a_min,i_a_max,i_b_min,i_b_max,i_c_min,i_c_max,i_d_min,i_d_max)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::interp_4D
! dummy arguments
    INTEGER,INTENT(in)::i_a_min,i_a_max
    INTEGER,INTENT(in)::i_b_min,i_b_max
    INTEGER,INTENT(in)::i_c_min,i_c_max
    INTEGER,INTENT(in)::i_d_min,i_d_max
    REAL(KIND=fp_1),INTENT(in),DIMENSION(i_a_min:i_a_max,i_b_min:i_b_max, &
      & i_c_min:i_c_max,i_d_min:i_d_max)::array
    REAL(KIND=fp_1),INTENT(inout)::a
    REAL(KIND=fp_1),INTENT(inout)::b
    REAL(KIND=fp_1),INTENT(inout)::c
    REAL(KIND=fp_1),INTENT(inout)::d
    REAL(KIND=fp_1),INTENT(in)::a_max
    REAL(KIND=fp_1),INTENT(in)::b_max
    REAL(KIND=fp_1),INTENT(in)::c_max
    REAL(KIND=fp_1),INTENT(in)::d_max
! local variables
    REAL(KIND=fp_1)::i_a,i_b,i_c,i_d
    REAL(KIND=fp_1)::a1,a2
    REAL(KIND=fp_1)::b1,b2
    REAL(KIND=fp_1)::c1,c2
    REAL(KIND=fp_1)::d1,d2
    INTEGER::i_a1,i_a2
    INTEGER::i_b1,i_b2
    INTEGER::i_c1,i_c2
    INTEGER::i_d1,i_d2

! *** calculate grid points enclosing the passes point coordinates ***
! calculate integer values each point coordinate
    i_a = INT(a * (1.0 / a_max) * i_a_max)
    i_b = INT(b * (1.0 / b_max) * i_b_max)
    i_c = INT(c * (1.0 / c_max) * i_c_max)
    i_d = INT(d * (1.0 / d_max) * i_d_max)
! calculate:
! (a) bounding grid points along each dimension ('i_x1' and 'i_x2')
!     NOTE: if the possition of the point along any one dimensions is on or past the 
!           boundary of that dimension, then both grid points are set to the boundary grid point
! (b) bounding points of the interval containing the point in question ('x1' and 'x2')
!     NOTE: the interval between these two points is fixed at the resolution along that dimension 
!           of the look-up table, even if the point in question lies outside the bounding space 
!           (this is to prevent divide-by-zero errors in the interpolation)
! (c) if the position of the point in question falls outside of the table boundary along any dimension,
!     the value at the required point is estimated via a linear extrapolation 
!     using the last two points long that particular dimension 
! parameter 'a'
    IF (a >= a_max) THEN
      i_a1 = i_a_max - 1
      i_a2 = i_a_max
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE IF (a < (i_a_min * (a_max / i_a_max))) THEN
      i_a1 = i_a_min
      i_a2 = i_a_min + 1
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE IF (a < 0.0) THEN
      i_a1 = i_a - 1
      i_a2 = i_a
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE
      i_a1 = i_a
      i_a2 = i_a + 1
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ENDIF
! parameter 'b'
    IF (b >= b_max) THEN
      i_b1 = i_b_max - 1
      i_b2 = i_b_max
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE IF (b < (i_b_min * (b_max / i_b_max))) THEN
      i_b1 = i_b_min
      i_b2 = i_b_min + 1
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE IF (b < 0.0) THEN
      i_b1 = i_b - 1
      i_b2 = i_b
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE
      i_b1 = i_b
      i_b2 = i_b + 1
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ENDIF
! parameter 'c'
    IF (c >= c_max) THEN
      i_c1 = i_c_max - 1
      i_c2 = i_c_max
      c1   = (i_c1 - 1) * (1.0 / i_c_max) * c_max
      c2   = (i_c2 - 0) * (1.0 / i_c_max) * c_max
    ELSE IF (c < (i_c_min * (c_max / i_c_max))) THEN
      i_c1 = i_c_min
      i_c2 = i_c_min + 1
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ELSE IF (c < 0.0) THEN
      i_c1 = i_c - 1
      i_c2 = i_c
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ELSE
      i_c1 = i_c
      i_c2 = i_c + 1
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ENDIF
! parameter 'd'
    IF (d >= d_max) THEN
      i_d1 = i_d_max - 1
      i_d2 = i_d_max
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE IF (d < (i_d_min * (d_max / i_d_max))) THEN
      i_d1 = i_d_min
      i_d2 = i_d_min + 1
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE IF (d < 0.0) THEN
      i_d1 = i_d - 1
      i_d2 = i_d
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE
      i_d1 = i_d
      i_d2 = i_d + 1
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ENDIF

! *** return function value ***
! interpolate
! NOTE: see "Applied Numerical Methods with Software" by Nakamura for details of 1-D and 2-D interpolation
    interp_4D = (1.0 / ((a2-a1)*(b2-b1)*(c2-c1)*(d2-d1))) * &
      & ( &
      &   (a-a1)*(b-b1)*(c-c1)*(d-d1) * array(i_a2,i_b2,i_c2,i_d2) + &
      &   (a-a1)*(b-b1)*(c-c1)*(d2-d) * array(i_a2,i_b2,i_c2,i_d1) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d-d1) * array(i_a2,i_b2,i_c1,i_d2) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d2-d) * array(i_a2,i_b2,i_c1,i_d1) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d-d1) * array(i_a2,i_b1,i_c2,i_d2) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d2-d) * array(i_a2,i_b1,i_c2,i_d1) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d-d1) * array(i_a2,i_b1,i_c1,i_d2) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d2-d) * array(i_a2,i_b1,i_c1,i_d1) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d-d1) * array(i_a1,i_b2,i_c2,i_d2) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d2-d) * array(i_a1,i_b2,i_c2,i_d1) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d-d1) * array(i_a1,i_b2,i_c1,i_d2) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d2-d) * array(i_a1,i_b2,i_c1,i_d1) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d-d1) * array(i_a1,i_b1,i_c2,i_d2) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d2-d) * array(i_a1,i_b1,i_c2,i_d1) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d-d1) * array(i_a1,i_b1,i_c1,i_d2) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d2-d) * array(i_a1,i_b1,i_c1,i_d1) &
      & )

!!$print*,"END *** return function value ***"

  END FUNCTION interp_4D
! **********************************************************


! **********************************************************
! *** general linear 5 dimensional interpolation routine ***
! **********************************************************
  FUNCTION interp_5D(array,a,b,c,d,e,a_max,b_max,c_max,d_max,e_max, &
    & i_a_min,i_a_max,i_b_min,i_b_max,i_c_min,i_c_max,i_d_min,i_d_max,i_e_min,i_e_max)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::interp_5D
! dummy arguments
    INTEGER,INTENT(in)::i_a_min,i_a_max
    INTEGER,INTENT(in)::i_b_min,i_b_max
    INTEGER,INTENT(in)::i_c_min,i_c_max
    INTEGER,INTENT(in)::i_d_min,i_d_max
    INTEGER,INTENT(in)::i_e_min,i_e_max
    REAL(KIND=fp_1),INTENT(in),DIMENSION(i_a_min:i_a_max,i_b_min:i_b_max, &
      & i_c_min:i_c_max,i_d_min:i_d_max,i_e_min:i_e_max)::array
    REAL(KIND=fp_1),INTENT(inout)::a
    REAL(KIND=fp_1),INTENT(inout)::b
    REAL(KIND=fp_1),INTENT(inout)::c
    REAL(KIND=fp_1),INTENT(inout)::d
    REAL(KIND=fp_1),INTENT(inout)::e
    REAL(KIND=fp_1),INTENT(in)::a_max
    REAL(KIND=fp_1),INTENT(in)::b_max
    REAL(KIND=fp_1),INTENT(in)::c_max
    REAL(KIND=fp_1),INTENT(in)::d_max
    REAL(KIND=fp_1),INTENT(in)::e_max
! local variables
    REAL(KIND=fp_1)::i_a,i_b,i_c,i_d,i_e
    REAL(KIND=fp_1)::a1,a2
    REAL(KIND=fp_1)::b1,b2
    REAL(KIND=fp_1)::c1,c2
    REAL(KIND=fp_1)::d1,d2
    REAL(KIND=fp_1)::e1,e2
    INTEGER::i_a1,i_a2
    INTEGER::i_b1,i_b2
    INTEGER::i_c1,i_c2
    INTEGER::i_d1,i_d2
    INTEGER::i_e1,i_e2
    
! *** calculate grid points enclosing the passes point coordinates ***
! calculate integer values each point coordinate
    i_a = INT(a * (1.0 / a_max) * i_a_max)
    i_b = INT(b * (1.0 / b_max) * i_b_max)
    i_c = INT(c * (1.0 / c_max) * i_c_max)
    i_d = INT(d * (1.0 / d_max) * i_d_max)
    i_e = INT(e * (1.0 / e_max) * i_e_max)
! calculate:
! (a) bounding grid points along each dimension ('i_x1' and 'i_x2')
!     NOTE: if the possition of the point along any one dimensions is on or past the 
!           boundary of that dimension, then both grid points are set to the boundary grid point
! (b) bounding points of the interval containing the point in question ('x1' and 'x2')
!     NOTE: the interval between these two points is fixed at the resolution along that dimension 
!           of the look-up table, even if the point in question lies outside the bounding space 
!           (this is to prevent divide-by-zero errors in the interpolation)
! (c) if the position of the point in question falls outside of the table boundary along any dimension,
!     the value at the required point is estimated via a linear extrapolation 
!     using the last two points long that particular dimension 
! parameter 'a'
    IF (a >= a_max) THEN
      i_a1 = i_a_max - 1
      i_a2 = i_a_max
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE IF (a < (i_a_min * (a_max / i_a_max))) THEN
      i_a1 = i_a_min
      i_a2 = i_a_min + 1
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE IF (a < 0.0) THEN
      i_a1 = i_a - 1
      i_a2 = i_a
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ELSE
      i_a1 = i_a
      i_a2 = i_a + 1
      a1   = i_a1 * (1.0 / i_a_max) * a_max
      a2   = i_a2 * (1.0 / i_a_max) * a_max
    ENDIF
! parameter 'b'
    IF (b >= b_max) THEN
      i_b1 = i_b_max - 1
      i_b2 = i_b_max
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE IF (b < (i_b_min * (b_max / i_b_max))) THEN
      i_b1 = i_b_min
      i_b2 = i_b_min + 1
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE IF (b < 0.0) THEN
      i_b1 = i_b - 1
      i_b2 = i_b
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ELSE
      i_b1 = i_b
      i_b2 = i_b + 1
      b1   = i_b1 * (1.0 / i_b_max) * b_max
      b2   = i_b2 * (1.0 / i_b_max) * b_max
    ENDIF
! parameter 'c'
    IF (c >= c_max) THEN
      i_c1 = i_c_max - 1
      i_c2 = i_c_max
      c1   = (i_c1 - 1) * (1.0 / i_c_max) * c_max
      c2   = (i_c2 - 0) * (1.0 / i_c_max) * c_max
    ELSE IF (c < (i_c_min * (c_max / i_c_max))) THEN
      i_c1 = i_c_min
      i_c2 = i_c_min + 1
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ELSE IF (c < 0.0) THEN
      i_c1 = i_c - 1
      i_c2 = i_c
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ELSE
      i_c1 = i_c
      i_c2 = i_c + 1
      c1   = i_c1 * (1.0 / i_c_max) * c_max
      c2   = i_c2 * (1.0 / i_c_max) * c_max
    ENDIF
! parameter 'd'
    IF (d >= d_max) THEN
      i_d1 = i_d_max - 1
      i_d2 = i_d_max
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE IF (d < (i_d_min * (d_max / i_d_max))) THEN
      i_d1 = i_d_min
      i_d2 = i_d_min + 1
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE IF (d < 0.0) THEN
      i_d1 = i_d - 1
      i_d2 = i_d
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ELSE
      i_d1 = i_d
      i_d2 = i_d + 1
      d1   = i_d1 * (1.0 / i_d_max) * d_max
      d2   = i_d2 * (1.0 / i_d_max) * d_max
    ENDIF
! parameter 'e'
    IF (e >= e_max) THEN
      i_e1 = i_e_max - 1
      i_e2 = i_e_max
      e1   = i_e1 * (1.0 / i_e_max) * e_max
      e2   = i_e2 * (1.0 / i_e_max) * e_max
    ELSE IF (e < (i_e_min * (e_max / i_e_max))) THEN
      i_e1 = i_e_min
      i_e2 = i_e_min + 1
      e1   = i_e1 * (1.0 / i_e_max) * e_max
      e2   = i_e2 * (1.0 / i_e_max) * e_max
    ELSE IF (e < 0.0) THEN
      i_e1 = i_e - 1
      i_e2 = i_e
      e1   = i_e1 * (1.0 / i_e_max) * e_max
      e2   = i_e2 * (1.0 / i_e_max) * e_max
    ELSE
      i_e1 = i_e
      i_e2 = i_e + 1
      e1   = i_e1 * (1.0 / i_e_max) * e_max
      e2   = i_e2 * (1.0 / i_e_max) * e_max
    ENDIF

! *** return function value ***
! interpolate
! NOTE: see "Applied Numerical Methods with Software" by Nakamura for details of 1-D and 2-D interpolation
    interp_5D = (1.0 / ((a2-a1)*(b2-b1)*(c2-c1)*(d2-d1)*(e2-e1))) * &
      & ( &
      &   (a-a1)*(b-b1)*(c-c1)*(d-d1)*(e-e1) * array(i_a2,i_b2,i_c2,i_d2,i_e2) + &
      &   (a-a1)*(b-b1)*(c-c1)*(d-d1)*(e2-e) * array(i_a2,i_b2,i_c2,i_d2,i_e1) + &
      &   (a-a1)*(b-b1)*(c-c1)*(d2-d)*(e-e1) * array(i_a2,i_b2,i_c2,i_d1,i_e2) + &
      &   (a-a1)*(b-b1)*(c-c1)*(d2-d)*(e2-e) * array(i_a2,i_b2,i_c2,i_d1,i_e1) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d-d1)*(e-e1) * array(i_a2,i_b2,i_c1,i_d2,i_e2) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d-d1)*(e2-e) * array(i_a2,i_b2,i_c1,i_d2,i_e1) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d2-d)*(e-e1) * array(i_a2,i_b2,i_c1,i_d1,i_e2) + &
      &   (a-a1)*(b-b1)*(c2-c)*(d2-d)*(e2-e) * array(i_a2,i_b2,i_c1,i_d1,i_e1) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d-d1)*(e-e1) * array(i_a2,i_b1,i_c2,i_d2,i_e2) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d-d1)*(e2-e) * array(i_a2,i_b1,i_c2,i_d2,i_e1) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d2-d)*(e-e1) * array(i_a2,i_b1,i_c2,i_d1,i_e2) + &
      &   (a-a1)*(b2-b)*(c-c1)*(d2-d)*(e2-e) * array(i_a2,i_b1,i_c2,i_d1,i_e1) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d-d1)*(e-e1) * array(i_a2,i_b1,i_c1,i_d2,i_e2) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d-d1)*(e2-e) * array(i_a2,i_b1,i_c1,i_d2,i_e1) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d2-d)*(e-e1) * array(i_a2,i_b1,i_c1,i_d1,i_e2) + &
      &   (a-a1)*(b2-b)*(c2-c)*(d2-d)*(e2-e) * array(i_a2,i_b1,i_c1,i_d1,i_e1) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d-d1)*(e-e1) * array(i_a1,i_b2,i_c2,i_d2,i_e2) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d-d1)*(e2-e) * array(i_a1,i_b2,i_c2,i_d2,i_e1) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d2-d)*(e-e1) * array(i_a1,i_b2,i_c2,i_d1,i_e2) + &
      &   (a2-a)*(b-b1)*(c-c1)*(d2-d)*(e2-e) * array(i_a1,i_b2,i_c2,i_d1,i_e1) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d-d1)*(e-e1) * array(i_a1,i_b2,i_c1,i_d2,i_e2) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d-d1)*(e2-e) * array(i_a1,i_b2,i_c1,i_d2,i_e1) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d2-d)*(e-e1) * array(i_a1,i_b2,i_c1,i_d1,i_e2) + &
      &   (a2-a)*(b-b1)*(c2-c)*(d2-d)*(e2-e) * array(i_a1,i_b2,i_c1,i_d1,i_e1) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d-d1)*(e-e1) * array(i_a1,i_b1,i_c2,i_d2,i_e2) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d-d1)*(e2-e) * array(i_a1,i_b1,i_c2,i_d2,i_e1) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d2-d)*(e-e1) * array(i_a1,i_b1,i_c2,i_d1,i_e2) + &
      &   (a2-a)*(b2-b)*(c-c1)*(d2-d)*(e2-e) * array(i_a1,i_b1,i_c2,i_d1,i_e1) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d-d1)*(e-e1) * array(i_a1,i_b1,i_c1,i_d2,i_e2) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d-d1)*(e2-e) * array(i_a1,i_b1,i_c1,i_d2,i_e1) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d2-d)*(e-e1) * array(i_a1,i_b1,i_c1,i_d1,i_e2) + &
      &   (a2-a)*(b2-b)*(c2-c)*(d2-d)*(e2-e) * array(i_a1,i_b1,i_c1,i_d1,i_e1) &
      & )

  END FUNCTION interp_5D
! **********************************************************


! *************************************************************************************************


END MODULE sue_lib

